﻿using System;
using System.IO;
using System.Linq;
using System.Net;


namespace RobvanderWoude
{
	internal class HTMLEscape
	{
		static readonly string progver = "1.01";


		static int Main( string[] args )
		{
			int rc = 0;

			int argumentscount = args.Length;
			if ( Console.IsInputRedirected )
			{
				argumentscount++;
			}
			if ( argumentscount != 1 || args.Contains( "/?" ) )
			{
				return ShowHelp( );
			}

			string unescapedtext = string.Empty;
			if ( Console.IsInputRedirected )
			{
				unescapedtext = Console.In.ReadToEnd( );
			}
			else if ( File.Exists( args[0] ) )
			{
				unescapedtext= File.ReadAllText( args[0] );
			}
			else
			{
				unescapedtext = args[0];
			}
			string escapedtext = WebUtility.HtmlEncode( unescapedtext );
			escapedtext = ConvertHTMLEntities( escapedtext );
			Console.Write( escapedtext );

			return rc;
		}



		static string ConvertHTMLEntities( string input )
		{
			string output = input.Replace( "&#39;", "'" );
			output = output.Replace( "&#192;", "&Agrave;" );
			output = output.Replace( "&3193;", "&Aacute;" );
			output = output.Replace( "&#194;", "&Acirc;" );
			output = output.Replace( "&#195;", "&Atilde;" );
			output = output.Replace( "&#196;", "&Auml;" );
			output = output.Replace( "&#197;", "&Aring;" );
			output = output.Replace( "&#198;", "&AElig;" );
			output = output.Replace( "&#199;", "&Ccedil;" );
			output = output.Replace( "&#200;", "&Egrave;" );
			output = output.Replace( "&#201;", "&Eacute;" );
			output = output.Replace( "&#202;", "&Ecirc;" );
			output = output.Replace( "&#203;", "&Euml;" );
			output = output.Replace( "&#204;", "&Igrave;" );
			output = output.Replace( "&#205;", "&Iacute;" );
			output = output.Replace( "&#206;", "&Icirc;" );
			output = output.Replace( "&#207;", "&Iuml;" );
			output = output.Replace( "&#208;", "&ETH;" );
			output = output.Replace( "&#209;", "&Ntilde;" );
			output = output.Replace( "&#210;", "&Ograve;" );
			output = output.Replace( "&#211;", "&Oacute;" );
			output = output.Replace( "&#212;", "&Ocirc;" );
			output = output.Replace( "&#213;", "&Otilde;" );
			output = output.Replace( "&#214;", "&Ouml;" );
			output = output.Replace( "&#216;", "&Oslash;" );
			output = output.Replace( "&#217;", "&Ugrave;" );
			output = output.Replace( "&#218;", "&Uacute;" );
			output = output.Replace( "&#219;", "&Ucirc;" );
			output = output.Replace( "&#220;", "&Uuml;" );
			output = output.Replace( "&#221;", "&Yacute;" );
			output = output.Replace( "&#222;", "&THORN;" );
			output = output.Replace( "&#223;", "&szlig;" );
			output = output.Replace( "&#224;", "&agrave;" );
			output = output.Replace( "&#225;", "&aacute;" );
			output = output.Replace( "&#226;", "&acirc;" );
			output = output.Replace( "&#227;", "&atilde;" );
			output = output.Replace( "&#228;", "&auml;" );
			output = output.Replace( "&#229;", "&aring;" );
			output = output.Replace( "&#230;", "&aelig;" );
			output = output.Replace( "&#231;", "&ccedil;" );
			output = output.Replace( "&#232;", "&egrave;" );
			output = output.Replace( "&#233;", "&eacute;" );
			output = output.Replace( "&#234;", "&ecirc;" );
			output = output.Replace( "&#235;", "&euml;" );
			output = output.Replace( "&#236;", "&igrave;" );
			output = output.Replace( "&#237;", "&iacute;" );
			output = output.Replace( "&#238;", "&icirc;" );
			output = output.Replace( "&#239;", "&iuml;" );
			output = output.Replace( "&#240;", "&eth;" );
			output = output.Replace( "&#241;", "&ntilde;" );
			output = output.Replace( "&#242;", "&ograve;" );
			output = output.Replace( "&#243;", "&oacute;" );
			output = output.Replace( "&#244;", "&ocirc;" );
			output = output.Replace( "&#245;", "&otilde;" );
			output = output.Replace( "&#246;", "&ouml;" );
			output = output.Replace( "&#248;", "&oslash;" );
			output = output.Replace( "&#249;", "&ugrave;" );
			output = output.Replace( "&#250;", "&uacute;" );
			output = output.Replace( "&#251;", "&ucirc;" );
			output = output.Replace( "&#252;", "&uuml;" );
			output = output.Replace( "&#253;", "&yacute;" );
			output = output.Replace( "&#254;", "&thorn;" );
			output = output.Replace( "&#255;", "&yuml;" );
			return output;
		}


		public static int ShowHelp( string errorMessage = "" )
		{
			#region Help Text

			/*
			HTMLEscape,  Version 1.00
			Display HTML-escaped input string in console
 
			Usage:  HTMLESCAPE  textfile
 
			   or:  HTMLESCAPE  unescapedstring
 
			   or:  some_program | HTMLESCAPE
 
			Where:  textfile         file containing text to be escaped
			        unescapedstring  string to be escaped
			        some_program     program whose output is to be escaped
 
			Note:   Exit code ("errorlevel") -1 in case of errors, otherwise 0.
	
			Written by Rob van der Woude
			https://www.robvanderwoude.com
			*/

			#endregion Help Text


			#region Error Message

			if ( !string.IsNullOrWhiteSpace( errorMessage ) )
			{
				Console.Error.WriteLine( );
				Console.ForegroundColor = ConsoleColor.Red;
				Console.Error.Write( "ERROR: " );
				Console.ForegroundColor = ConsoleColor.White;
				Console.Error.WriteLine( errorMessage );
				Console.ResetColor( );
			}

			#endregion Error Message


			#region Display Help

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "HTMLEscape,  Version {0}", progver );

			Console.Error.WriteLine( "Display HTML-escaped input string in console" );

			Console.Error.WriteLine( );

			Console.Error.Write( "Usage:  " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.WriteLine( "HTMLESCAPE  textfile" );
			Console.ResetColor( );

			Console.Error.WriteLine( );

			Console.Error.Write( "   or:  " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.WriteLine( "HTMLESCAPE  unescapedstring" );
			Console.ResetColor( );

			Console.Error.WriteLine( );

			Console.Error.Write( "   or:  " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.WriteLine( "some_program | HTMLESCAPE" );
			Console.ResetColor( );

			Console.Error.WriteLine( );

			Console.Error.Write( "Where:  " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "textfile" );
			Console.ResetColor( );
			Console.Error.WriteLine( "         file containing text to be escaped" );

			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "        unescapedstring" );
			Console.ResetColor( );
			Console.Error.WriteLine( "  string to be escaped" );

			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "        some_program" );
			Console.ResetColor( );
			Console.Error.WriteLine( "     program whose output is to be escaped" );

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "Note:   Exit code (\"errorlevel\") -1 in case of errors, otherwise 0." );

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "Written by Rob van der Woude" );

			Console.Error.WriteLine( "https://www.robvanderwoude.com" );

			#endregion Display Help


			return -1;
		}
	}
}
